@@ -5,18 +5,61 @@ from nltk.corpus import stopwords
 from nltk import download
 from gensim.models import KeyedVectors
 from sentence_transformers import SentenceTransformer, util, InputExample, losses, models
+from string import punctuation
+from nltk.stem import PorterStemmer
+from nltk.stem.isri import ISRIStemmer
 
+def preprocess(sentence, remove_punct, remove_stop_words, stemm, lang='en'):
+    if lang.lower() == 'en':
+        ps = PorterStemmer()
+        # remove punctuations
+        if not remove_punct:
+            sentence = sentence.translate(str.maketrans('', '', punctuation))
+        # remove stop words and stem
+        if remove_stop_words and stemm:
+            download('stopwords')
+            stop_words = stopwords.words('english')
+            return ' '.join([ps.stem(w) for w in sentence.lower().split() if w not in stop_words])
+        # stem only
+        elif not remove_stop_words and stemm:
+            return ' '.join([ps.stem(w) for w in sentence.lower().split()])
+        else:
+            # lower case and remove extra white spaces
+            return ' '.join([w for w in sentence.lower().split()])
+    elif lang.lower() == 'ar':
+        st = ISRIStemmer()
+        # remove punctuations
+        if not remove_punct:
+            sentence = sentence.translate(str.maketrans('', '', punctuation))
+        # remove stop words and stem
+        if remove_stop_words and stemm:
+            download('stopwords')
+            stop_words = stopwords.words('arabic')
+            return ' '.join([st.stem(w) for w in sentence.lower().split() if w not in stop_words])
+        # stem only
+        elif not remove_stop_words and stemm:
+            return ' '.join([st.stem(w) for w in sentence.lower().split()])
+        else:
+            # lower case and remove extra white spaces
+            return ' '.join([w for w in sentence.lower().split()])
+    else:
+        raise Exception('non recognized language please specify either en|ar')
+        
+        
 class sentence_tranformer():
     def __init__(self, source_names, target_names, model=None):
         self.source_names = source_names
         self.target_names = [target  for target in target_names if not (pd.isnull(target))] 
         self.model = model
-        self.encoded_targets = self.model.encode(self.target_names)
-        
         # if no model is provided use the default model
         if model is None:
+            print('initializing the model...')
             self.model = SentenceTransformer('paraphrase-MiniLM-L6-v2')
 
+        self.encoded_targets = self.model.encode(self.target_names)
+        
+
+
     def match(self, topn=1):
         '''
         Main match function. return only the top candidate for every source string.
@@ -28,9 +71,9 @@ class sentence_tranformer():
 
         return match_output
 
-
-    def normalize_data(self, data):
-        return (data - np.min(data)) / (np.max(data) - np.min(data))
+    def clean_data(self, remove_punct=True, remove_stop_words=True, stemm=False, lang='en'): 
+        self.source_names = [preprocess(sent, remove_punct, remove_stop_words, stemm, lang) for sent in self.source_names]
+        self.target_names = [preprocess(sent, remove_punct, remove_stop_words, stemm, lang) for sent in self.target_names]
 
 
     def max_cosine_sim(self, input):
@@ -83,16 +126,16 @@ class sentence_tranformer():
 
 
 class word_mover_distance():
-    def __init__(self, source_names, target_names, model=None):
+    def __init__(self, source_names, target_names, model):
         self.source_names = source_names
         self.target_names = target_names
         self.model = model
         
         # if no model is provided use the default model
-        if model is None:
-            self.model = KeyedVectors.load_word2vec_format('BioWordVec_PubMed_MIMICIII_d200.vec.bin', binary=True)
+        # if model is None:
+        #     self.model = KeyedVectors.load_word2vec_format('BioWordVec_PubMed_MIMICIII_d200.vec.bin', binary=True)
 
-    def match(self, topn):
+    def match(self, topn=1):
         '''
         Main match function. return only the top candidate for every source string.
         '''
@@ -105,18 +148,9 @@ class word_mover_distance():
         return match_output
 
 
-    # def clean_data(self):
-    #     download('stopwords') 
-    #     self.source_names = [self.preprocess(sent) for sent in self.source_names]
-    #     self.target_names = [self.preprocess(sent) for sent in self.target_names]
-
-
-    # def preprocess(self, sentence):
-    #     stop_words = stopwords.words('english')
-
-    #     sentence = sentence.translate(str.maketrans('', '', punctuation))
-
-    #     return [w for w in sentence.lower().split() if w not in stop_words]
+    def clean_data(self, remove_punct=True, remove_stop_words=True, stemm=False, lang='en'): 
+        self.source_names = [preprocess(sent, remove_punct, remove_stop_words, stemm, lang) for sent in self.source_names]
+        self.target_names = [preprocess(sent, remove_punct, remove_stop_words, stemm, lang) for sent in self.target_names]
 
 
     def min_wmd_distance(self, input):
